# ----------------------------------------------------------------------
'''
 script     : l4c_gen_Ancillary.py
 invocation : l4c_gen_Ancillary_v7.3.0.py SMAP_L4_C_Ancillary_v7.3.0.h5 SMAP_L4_C_Ancillary_INPUTS_v7.3.0.lis SMAP_BPLUT_2017-05-27.csv SMAP_MD5s.csv
               Example: <AncillaryHdf5Path> <inputListPath> <BPLUT_Path> <MD5_Path> <doyLut.csv> <CompressLevel={4}> 
 author     : joe glassy, Lupine Logic Inc.
maintainer  : toby kundig, NTSG
 version    : 7.4.1-2021-09-15T10:17:00, updated BPLUT for new version, small errors recorded in history
 version    : 7.4.1-2020-08-05T13:41:00, updated BPLUT for NRv8.3
 version    : 7.4.1-2020-07-10T16:52:00, updated for new BPLUT, for v5 Delivery NRv8.1
 version    : 7.4.0-2018-05-22T09:30:00, new BPLUT, recal/spinup for Natv72 delivery
 version    : 7.3.0-2017-06-14T11:38:00, updated to correct very minor omissions for v7.3.0 delivery, add MD5s text 
              new file level UUID is: 20831c34-5122-11e7-bcf5-2bd2c7411446
 version    : 7.3.0-2017-06-07T14:52:00, updated for major v7.3.0 delivery
 version    : 7.0-2015-09-07T19:55, updated metadata only (PSD 1.8.1) etc
 version    : 7.0-2015-08-20T11:30, updated for new Aug 10 spinup, new BPLUT and SOC
 version    : 6.0-2015-06-04T10:30, updated to reflect Release tag R11-Y2
 version    : 6.0-2015-05-21T16:24:00, updated for new BPLUT, SMAP_BPLUT_2015-05-21.csv for Nature Run v4 spin up delivery.
 version    : 6.0-2015-05-20T15:15:00, updated for Nature Run v4 spin up delivery.
              Note that the BPLUT is still left outside as an external text file.
 version    : 5.0.1.7-2014-11-11, added QA legend and continued attribute name harmonization across all components
 version    : 5.0.1.5-2014-10-08T09:05:00, continued attribute name harmonization across all components
 history    : v7.4.1-2021-09-15T12:36:00, Updated read_store_BPLUT and MD5s array to np.array with variable dataty              pe to be passed to createdataset function
 history    : v5.0.1.4-2014-08-19T14:24:00, added 1km lat, lon images
 history    : v5.0.1.3-2014-08-13T13:23:00, changed delimited to ":" for timestamp, and
              scheme to store(3) "virgin" SOC layers stores in separate SMAP_L4_C_SOC_DAY1.h5 
              file. The dynamic (updated) (2) (NatureRun) rootzone soil moisture layers are now stored
              in the SOC file, SMAP_L4_C_SOC.h5 since they are no longer static.
 history    : v5.0.1.2-2014-06-25T15:47:00-07:00, added creation of UMT_META group, to 
              facilitate future storage of essentially static parameters that may not
              really need to go into the RIP file for {l4c_DAILY.ex, l4c_MET.ex, or l4c_MODIS.ex}
 history    : v5.0.2-2014-06-02T12:20:00-07:00, production date update only
 history    : v5.0.1-2014-05-20T16:14:00-07:00, minor metadata cleanups
 history    : v5.0.1-2014-04-14, latest publicly distributed ancillary release
 history    : v5.0.1-2014-02-13T12:06:00-07:00, as D5 prep, added "L4C_M01land_PixFilter.uint8 layer"
             as well as new MERRA_climate_M09_gwetroot_min.flt32, MERRA_climate_M09_gwetroot_max.flt32
 history    : 2013-12-02T12:27:11:00, added 3km MASK
 notes      : For v7.3.0 note: L4_C_SUBSYSTEM_OPS_v7-2016-05-27T14:50:00
 notes      : The BPLUT v7.3.0 UUID is: "f59e8d14-4a19-11e7-b073-1396440ba582"
 purpose    : Create a fresh SMAP_L4_C_Ancillary.h5 file, takes about 133 seconds on wheat

 dependencies : Python 2.7 or above with h5py and uuid package, and bplut_ingest.ex executable

 Notes      : In the future we MAY ingest the BPLUT using a special C program, BPLUT_ingest.c
              but this is still not fully enabled as of v6 (5/20/2015).
              It would produce an initial SMAP_L4_C_Ancillary.h5 file to which the rest of these
              DATASETS and attributes are added.
#
   Table of contents:
   --EASEGRID 1km,3km,9km masks, PFT 1km, tile index lookups,and all related images,etc
   --doy_to_period LOOKUP Table: a table of 46 rows with two columns
     or, implement as a doy_to_period[46] 1D vector with the index being the (period-1) and
     the value being the Day-of-Year value
   --BPLUT (optional), usually now a separate text file
#
 UUID seed used for BPLUT
  uuidSeed = "www.ntsg.umt.edu/NASA/SMAP/L4_C/BPLUT-2013-09-08T" + str(time.time())
  bplutUUID = uuid.uuid5(uuid.NAMESPACE_URL,uuidseed)

  LDOM1KM is 134787240
  LDOM9KM is 1664040
 
  PFT Type 5 Land Cover Classification
        water = 0
        ENF evergreen needleleaf tree = 1
        evergreen broadleaf tree = 2
        deciduous needleleaf tree = 3
        deciduous broadleaf tree = 4
        shrub = 5
        grass = 6
        cereal crop = 7
        broadleaf crop = 8
        urban and built-up = 9
        snow and ice = 10
        barren or sparsely vegetated = 11
        unclassified = 254

'''
# ----------------------------------------------------------------------

AppVers = "7.4.1-2021-09-15T10:15:00"
VersId  = "v7.4.1"
AppName = "l4c_gen_Ancillary"

BPLUT_DATESTMP   = "2021-09-08T15:34:00"
BPLUT_UUID       = "4270d56e-2f2d-4c54-839c-4c19d5f8dd79"
BPLUT_MD5        = "cc2ea579966ab982ab74cd9cad20fe1f"

L4C_OPSSTREAM    = "Vv60XX"
# may evolve to R11-G 
L4C_OPSRELEASE   = "TBD"
L4CSUBSYSTEM_TAG = "L4_C_SUBSYSTEM_OPS_v7.4.1-2021-09-15T10:35:00"

# ----------------------------------------------------------------------
import os, sys
import datetime, uuid
import time, random

import numpy as np
import h5py


# ---------------------------------------------------------------------
def logMsg(logf,msg):
    '''
    logMsg()
    inputs: logf -- open log file handle
            msg -- formatted text message to output to log file and console
    URL: for task logging techniques see:
     https://docs.python.org/dev/howto/logging-cookbook.html#logging-to-a-single-file-from-multiple-processes            
    '''
    logf.write(msg+"\n")
    if logf != sys.stdout:
        print(msg)
    return 0
    # end::logMsg()
    
# ----------------------------------------------------------------------
def generate_uuid(granulePath,thisDOI=None):
    """
    generate_uuid()
    Inputs: 
        granulePath -- a basename of a file to generate this UUID for
    Methods:
        1) we use Python module UUID
        2) we use the uuid.uuid5() which generates a SHA-1 hash on the
        namespace plus the user provided seed information, such as a URI or URL
    Note: DOI for L4_C is: DOI_Id = "10.5067/22TFAUSNLO9R"
    """
    uuidString = ""
    DOI_Id = "10.5067/22TFAUSNLO9R"
    #
    if thisDOI is None:
        DOI_Id = "10.5067/22TFAUSNLO9R"
    # Joe Ardizzone provided this L4_C specific DOI on 6/27/2014, "10.5067/22TFAUSNLO9R"
    if(len(granulePath) <= 1):
        granulePath = "SMAP_L4_C_mdl_20150430T000000_V10001_001.h5"
    # we devised this URI seed just as an example...
    uuidSeed = "www.ntsg.umt.edu/SMAP/L4_C/ANC/" + granulePath + "/"+ str(time.time()) + "/DOI="+ DOI_Id
    granuleUUID = str(uuid.uuid5(uuid.NAMESPACE_URL,uuidSeed))
    uuidString = granuleUUID
    # return UUID as a string
    return uuidString
    # end::generate_uuid()

# ----------------------------------------------------------------------    
def year_DOY_to_Date(yearIdx,doyIdx):
    '''
    function: year_DOY_to_Date(year,doy)
    inputs  : integer yearIdx, integer day-of-year, doyIdx 1..366
    purpose : Convert an integer year YYYY and day of year (yearday) 1..366 to
              a date in YYYY-MM-DD format.
    dependencies : import datetime    
    returns : string, in YYYY-MM-DD format
    notes   : Example:
        rslt = datetime.datetime.strptime('2010 120', '%Y %j')
        rsltDate = str(rslt)[0:10]
        
    notes   : This also works, assuming yr = 2001 and doy = 120
    date.fromordinal(date(yr,1,1).toordinal()+ doy -1)
          Out[14]: datetime.date(2001, 4, 30)

    '''
    thisDate = "UNDEFINED*DATE"
    # construct input string in "YYYY DOY" format
    inputDate = str(yearIdx) + " " + str(doyIdx).zfill(3)
    try:
        # Return 'thisDate' as string 'YYYY-MM-DD'
        thisDate = str(datetime.datetime.strptime(inputDate, '%Y %j'))[0:10]
    except:
        pass
    return thisDate

# ----------------------------------------------------------------------        
def date_to_DOY(yearIdx,monthIdx,dayOfMonthIdx):
    '''
    date_to_DOY() -- given a date in YYYY-MM-DD format, generate a DOY index {1..366}
    dayOfYear = (date.toordinal(date(2001,4,30)) - date.toordinal(date(2001,1,1)))+1
       Out[24]: 0
    # Note: this works when the 'yearIdx' stays 
    '''
    dayOfYear = toordinal(date(yearIdx,monthIdx,dayOfMonthIdx)) - date.toordinal(date(yearIdx,1,1))+1
    # Return the day-of-year integer value {1..365} if a common year, or {1..366} if a leap year
    return dayOfYear 
    # end:: date_to_DOY()
    
# ----------------------------------------------------------------------
def genISO8601_datetime(myTime=None,suppressSeconds=None,includeTimezone=None):
    """
    function: genISO8601_datetime()
    purpose : Generates a string, ISO 8601 compliant datetime for the current time.
      time.struct_time(tm_year=2012, tm_mon=4, tm_mday=2, tm_hour=14, tm_min=49,
       tm_sec=51, tm_wday=0, tm_yday=93, tm_isdst=1)
    inputs  : time.localtime()|None, True|False, True|False    
    returns : a date-string like: YYYYMMDDTThh:MM:ss
    notes   : includeTimeZone option is not yet implemented
    notes   : 2014-10-08, Joe Ardizzone needs ISO8601 form YYYY-MM-DDThh:mm:%s.000Z
    
 // <JMG> 2014-09-12, now returns UTC in NASA SMAP preferred format
        // that includes a dummy sub-second interval qualifier prior to the 'Z'
        strcpy(fmtSpecStr,"%Y-%2m-%2dT%H:%M:%S.000Z" );
        strftime(utc_dtStr_p, 64-1,fmtSpecStr,gm_tmPtr);
        strftime(tzBuf1,15,"%H", gm_tmPtr);

    """
    timeStmpDelimiter = ':'
    if myTime is None:
        rNow = time.localtime()
    else:
        rNow = myTime
    # First, generate the YYYY-MM-DD segment
    yearday = str(rNow.tm_year).zfill(4) +"-"+str(rNow.tm_mon).zfill(2)+"-"+str(rNow.tm_mday).zfill(2)
    # next, generate and glue on the 'T' and timestamp HH-MM segment (use dash, to avoid using illegal ':' delimiter)
    # <JMG> modified to conform to required ISO8601 Joe A needs, ending with ".000Z"
    timeStmp= "T"+str(rNow.tm_hour).zfill(2) + timeStmpDelimiter + str(rNow.tm_min).zfill(2) +".000Z"
    # Now by default we eanble the seconds component, and only suppress it when requested
    if suppressSeconds is None:
        timeStmp = timeStmp + timeStmpDelimiter + str(rNow.tm_sec).zfill(2)

    # Note: the timezone qualification string is currently omitted.
    iso8601Datetime = yearday + timeStmp
    # return YYYYMMDDThhMM{ss}
    return iso8601Datetime 
    # end::genISO8601_datetime()

# ----------------------------------------------------------------------
def createFileAttributes(hObj,thisGranuleName,logf):
    '''
    createFileAttributes()
    purpose: create file level attributes for SMAP_L4_C_Ancillary.h5, 
    where these are currently written into the ROOT "/" group.
    
    '''
    logMsg(logf,"@A createFileAttributes: updating file attributes for %s"%(thisGranuleName))
    
    # Create file level attributes (of type string)
    hObj.attrs['TITLE']     = "SMAP L4_C Ancillary data -- EASEGRID2 1km (MO1),3km (M03) and 9km(M09) masks and land cover definitions"
    hObj.attrs['MISSION'  ] = "NASA SMAP Mission, Operational phase"
    hObj.attrs['PROJECT'  ] = "NASA SMAP Level 4 Carbon Model (L4_C) daily,global 9km"
    hObj.attrs['DELIVERY_VERSION'  ] = VersId
    
    # INCLUDE new MANDATORY subsystem release tag
    hObj.attrs['L4_C_SUBSYSTEM_RELEASE'  ] = L4CSUBSYSTEM_TAG
    
    hObj.attrs['L4_C_SW_VERSION'         ] = AppVers
    hObj.attrs['L4_C_OPS_STREAM'         ] = L4C_OPSSTREAM
    hObj.attrs['L4_C_OPS_RELEASE'        ] = L4C_OPSRELEASE
    
    hObj.attrs['L4_C_DATASET_VERSION'    ] = VersId
    hObj.attrs['L4_C_DATASET_DATETIME'   ] = AppVers
    hObj.attrs['L4_C_SW_DATETIME'        ] = genISO8601_datetime()
    hObj.attrs['L4_C_PRODUCTION_DATETIME'] = genISO8601_datetime()
    hObj.attrs['L4_C_ANCILLARY_GRANULE' ]  = hObj.filename
    
    hObj.attrs['SMAP_L4_C_PSD_VERS'     ] = "1.8.1"
    hObj.attrs['SMAP_L4_C_PSD_DATE'     ] = "2015-06-22"
    
    hObj.attrs['L4CMET_FT_SOURCE_OPTIONS' ] = "(GEOS5_TS|ACTIVE_RADAR|ACTIVE_TSURF|PASSIVE_RADIOMETER)"

    # DOI is assigned for main L4CMDL
    hObj.attrs['L4_C_MDL_DOI'           ] = "10.5067/22TFAUSNLO9R"
    # DOI not yet assigned for SMAP_L4_C_Ancillary file itself
    hObj.attrs['L4_C_ANC_DOI'           ] = "DOI TBD for SMAP_L4_C_Ancillary.h5"
    
    hObj.attrs['granule_UUID' ]  = generate_uuid(thisGranuleName)
    
    hObj.attrs['software_used_to_generate'     ] = sys.argv[0]
    hObj.attrs['software_author'               ] = "Joe Glassy,NASA SMAP L4_C team"
    hObj.attrs['component'] = "Plant Function Type (PFT) types are used"
    
    hObj.attrs['production_poc'         ] = "Joe Glassy, Univ. Montana L4_C lead software engineer"
    hObj.attrs['research_science_PI'    ] = "Dr. John Kimball, NTSG, Univ. Montana"
    hObj.attrs['research_algorithm'     ] = "tcf_spin_v3.4,laj"
    hObj.attrs['research_code_author'   ] = "Lucas Jones, Univ. Montana"
    #
    hObj.attrs['projection_name'    ] = "EASEGRID v2"
    
    hObj.attrs['projection_1km_rows'           ] = nRows1km
    hObj.attrs['projection_1km_columns'        ] = nCols1km
    hObj.attrs['projection_3km_rows'           ] = nRows3km
    hObj.attrs['projection_3km_columns'        ] = nCols3km
    hObj.attrs['projection_9km_rows'           ] = nRows9km
    hObj.attrs['projection_9km_columns'        ] = nCols9km
    
    # store counts of 1D sparse vector pixels for 1KM,9KM res
    # LDOM1KM is 134787240
    # LDOM9KM is 1664040
    # LDOM3KM = 56359296
    #
    hObj.attrs['LDOM1KM_1D_SparsePix' ] = LDOM1KM
    hObj.attrs['LDOM3KM_1D_SparsePix' ] = LDOM3KM
    hObj.attrs['LDOM9KM_1D_SparsePix' ] = LDOM9KM
    #
    # Fetch name of host we built this on, via uname -a parts...
    hObj.attrs['data_source_host'   ] = os.uname()[1]
    hObj.attrs['BPLUT_DATETIME'     ] = "2021-09-08"
    hObj.attrs['BPLUT_version'      ] = "2020-09-01"
    hObj.attrs['BPLUT_UUID'         ] = BPLUT_UUID
    hObj.attrs['BPLUT_MD5'          ] = BPLUT_MD5
    hObj.attrs['ANC_FILE_UUID'      ] = generate_uuid("SMAP_L4_C_Ancillary.h5","www.ntsg.umt.edu/SMAP/ANC")
    
    # Include the QA legend as a convenience
    hObj.attrs['QA_BITFLAG_L01_HDR1'  ] = 'QA    Bitflag      Bit Position(s)  N.    Value        Description'
    hObj.attrs['QA_BITFLAG_L02_HDR2'  ] = 'flag  Name         (Start,End)      Bits  Range'
    hObj.attrs['QA_BITFLAG_L03_HDR3'  ] = '------------------------------------------------------------------------------------------'
    hObj.attrs['QA_BITFLAG_L04_FLAG01']="01    NEE bit       00 - 00         (1)   {0|1}        0=NEE within valid range,1=out of range"
    hObj.attrs['QA_BITFLAG_L04_FLAG02']="02    GPP bit       01 - 01         (1)   {0|1}        0=GPP within valid range,1=out of range"
    hObj.attrs['QA_BITFLAG_L04_FLAG03']="03    Rh  bit       02 - 02         (1)   {0|1}        0=Rh  within valid range,1=out of range"
    hObj.attrs['QA_BITFLAG_L04_FLAG04']="04    SOC bit       03 - 03         (1)   {0|1}        0=SOC within valid range,1=out of range"
    hObj.attrs['QA_BITFLAG_L04_FLAG05']="05    PFT dominant  04 - 07         (4)   {1..8}       Dominant PFT class, decimal integer {1..8} inclusive"
    hObj.attrs['QA_BITFLAG_L04_FLAG06']="06    QA score      08 - 11         (4)   {0,1,2,3}    0=best,3=worst; overall Quality score"
    hObj.attrs['QA_BITFLAG_L04_FLAG07']="07    GPP method    12 - 12         (1)   {0|1}        0=used MODIS FPAR,1=used FPAR climatology"
    hObj.attrs['QA_BITFLAG_L04_FLAG08']="08    NDVI method   13 - 13         (1)   {0|1}        0=used FPAR or FPAR clim, 1=used NDVI"
    hObj.attrs['QA_BITFLAG_L04_FLAG09']="09    F/T method    14 - 14         (1)   {0|1}        0=used RADAR F/T, 1=used Tsurf F/T"
    hObj.attrs['QA_BITFLAG_L04_FLAG10']="10    IsFill        15 - 15         (1)   {0|1}        0=is NOT a Fillvalue, 1=Is a Fillvalue" 

    add_PFT_Legend(hObj)
    # end::createFileAttributes()


# ----------------------------------------------------------------------
def add_PFT_Legend(hObj):
    '''
    function: add_PFT_legend() -- optionally, add PFT legend. This is 
    now currently OFF by default since it appears in the SMAP_L4_C_Ancillary.h5
    file instead.
    '''
    # we repeat the PFT classification as a file-level attribute set for user convenience
    hObj.attrs['land_cover_classification' ] = "PFT Plant Function Type (PFT) from MODIS MCD Type-5"
    hObj.attrs['PFT_00_Water '        ] = "00 water"
    hObj.attrs['PFT_01_ENF'           ] = "01 ENF evergreen needleleaf tree"
    hObj.attrs['PFT_02_EBF'           ] = "02 EBF evergreen broadleaf tree"
    hObj.attrs['PFT_03_DNF'           ] = "03 DNF deciduous needleleaf tree"
    hObj.attrs['PFT_04_DBF'           ] = "04 DBF deciduous broadleaf tree"
    hObj.attrs['PFT_05_Shrub'         ] = "05 Shrub"
    hObj.attrs['PFT_06_Grass'         ] = "06 Grass"
    hObj.attrs['PFT_07_Cereal_crop'   ] = "07 Cereal crop"
    hObj.attrs['PFT_08_Broadleaf_crop'] = "08 Broadleaf crop"
    hObj.attrs['PFT_09_Urban'         ] = "09 Urban and built-up landuse (NOT-USED)"
    hObj.attrs['PFT_10_SNOW_ICE'      ] = "10 Snow and ice (NOT-USED)"
    hObj.attrs['PFT_11_BARREN  '      ] = "11 Barren or sparsely vegetated (NOT-USED)"
    hObj.attrs['PFT_254_UNCLASSIFIED' ] = "254 Unclassified (NOT-USED)"
    # end:: add_PFT_legend()


# ----------------------------------------------------------------------
def create_doy_lut(hObj,doyLutPath,logf):
    '''
    function: create_doy_lut()
    inputs  : hObj -- HDF5 open file handle to write to
            : doyLutPath -- full path to day of year lookup table
            : logf -- handle of open session log file
    purpose:  This function creates a DOY-to-Period Index lookup table
    revised: 2017-06-07T16:01
    status : Needs to be re-tested and certified.
    notes  : 2014-05-20T16:28:00, now in process of extending to a 3-column file
    storing CSV columns "day-of-year","composite-8day-period","composite-16day-period"
    '''
    doyLutName= "doy_to_period_lut"
    nDoyLutRows = 46 
    doyLutAry = np.zeros(nDoyLutRows,'int16')
    #
    if not os.path.exists(doyLutPath):
        print("create_doy_lut:: ERROR: cannot open doy_to_period_index LUT %s"%(doyLutPath))
        # sys.exit(-1)
        return doyLutAry 
    #
    doyLutLis=[]
    with open(doyLutPath) as inf:
        doyLutLis = [lutRec.strip('\n\r') for lutRec in inf if not ln.startswith('#')]
    k=0
    for dRec in doyLutLis:
        print("dRec %d --> %s"%(k,repr(dRec)))
        dRec = dRec.split(',')
        doy = int(dRec[0])
        comp_8dayPeriod = int(dRec[1])
        comp_16dayPeriod = int(dRec[2])
        doyLutAry[k] = doy
        # print("k %d DOY %d PERIOD %d"%(k,doy,compPeriod))             
        k += 1

    print("Creating DOYLUT (%s) with rows %d <%s>"%(doyLutName,nDoyLutRows,repr(doyLutAry)))
    print("BROKEN UNTIL re-integrated as 3-column dataset")
    sys.exit(-1)
    # doyLutObj = hObj.create_dataset(doyLutName,shape=doyLutAry.shape,dtype='int16',data=doyLutAry)
    return doyLutAry
    # end::create_doy_lut()


# ----------------------------------------------------------------------
def createUM_META_Attributes(hObj):
    '''
    function : createUM_META_Attributes
    purpose  : writes L4_C specific static parameters
      into the ./UM_META group of SMAP_L4_C_Ancillary.h5 ancillary file
     This is an initial set to which others will be added, eventually
      enabling these to be moved out of the RIP file. They are not
      yet read from /UM_META, but currently exist as placeholders and
      as backups for next generation development.
     notes    : the source of these name-value-pair information will be
      migrated to a RIP file used by this Python utility (l4c_gen_ancillary.py)
     revised: 2014-08-08T10:10:00,jmg    
      
    '''
    # fetch the UMT_META group reference
    umMetaGrp = hObj["UMT_META"]
    # now, write attributes to it.
    umMetaGrp.attrs["L4C_INPUT_NEE_VALID_MIN"]             = -30.0
    umMetaGrp.attrs["L4C_INPUT_L4C_INPUT_NEE_VALID_MAX"]   = 20.0
    umMetaGrp.attrs["L4C_INPUT_L4C_INPUT_GPP_VALID_MIN"]   =  0.0
    umMetaGrp.attrs["L4C_INPUT_L4C_INPUT_GPP_VALID_MAX"]   = 30.0
    umMetaGrp.attrs["L4C_INPUT_L4C_INPUT_RH_VALID_MIN"]    =  0.0
    umMetaGrp.attrs["L4C_INPUT_L4C_INPUT_RH_VALID_MAX"]    = 20.0
    umMetaGrp.attrs["L4C_INPUT_L4C_INPUT_SOC_VALID_MIN"]   =  0.0
    umMetaGrp.attrs["L4C_INPUT_L4C_INPUT_SOC_VALID_MAX"]   = 25000.0
    umMetaGrp.attrs["L4C_INPUT_L4C_INPUT_GPP_METHOD_THRESHOLD"] = 50.0 
    umMetaGrp.attrs["L4C_INPUT_L4C_INPUT_NDVI_METHOD_THRESHOLD"] = 50.0 
    # 

    # end:: createUM_META_Attributes()


# ----------------------------------------------------------------------
def createNewHdf5Dataset(hObj,srcPath,cmpLev,ithInput,logf):
    '''
    function: createNewHdf5Dataset(hObj,curPath,cmpLev) 
    purpose : create (next) new dataset by ingesting 'srcPath', using compression level cmpLev
    notes   : n/a
    '''
    # n. of expected pixels in sparse 1D vectors
    nLDOM1KM = 134787240
    nLDOM3KM = 56359296
    nLDOM9KM = 1664040
    #
    LDOM1KM = 134787240
    LDOM9KM = 1664040
    LDOM3KM = 56359296
    
    # Construct dataasetname from just the basename part
    basenameData = os.path.basename(srcPath)

    if basenameData.find("land_") >= 0:
        nRows = 1
        if basenameData.find("M01") >= 0:
            nCols = nLDOM1KM
        elif basenameData.find("M03") >= 0:
            nCols = nLDOM3KM
        elif basenameData.find("M09") >= 0:
            nCols = nLDOM9KM
    else:
        if basenameData.find("M01") >= 0:
            nRows = 14616
            nCols = 34704
        elif basenameData.find("M03") >= 0:
            nRows = 4872
            nCols = 11568
        elif basenameData.find("M09") >= 0:     
            nRows = 1624
            nCols = 3856
        else:
            nRows = 1624
            nCols = 3856

    nElem = nRows * nCols
        
    if srcPath.find(".uint8") >= 0:
        hDatasetName= basenameData.replace('.uint8','_uint8')
        dataType = 'uint8'
    elif srcPath.find(".int16")>= 0:
        hDatasetName= basenameData.replace('.int16','_int16')
        dataType = 'int16'      
    elif srcPath.find(".uint16")>= 0:
        hDatasetName= basenameData.replace('.uint16','_uint16')
        dataType = 'uint16'     
    elif srcPath.find(".int32")>= 0:
        hDatasetName= basenameData.replace('.int32','_int32')
        dataType = 'int32'
    elif srcPath.find(".uint32")>= 0:
        hDatasetName= basenameData.replace('.uint32','_uint32')
        dataType = 'uint32'
    elif srcPath.find(".flt32")>= 0:
        hDatasetName= basenameData.replace('.flt32','_flt32')
        dataType = 'float32'
    elif srcPath.find(".flt64")>= 0:
        hDatasetName= basenameData.replace('.flt64','_flt64')
        dataType = 'float64'
    else:
        printf("Supported datatype in %s was NOT specified!\n"%(srcPath))
        sys.exit(-1)

    # Create PFT group ONLY if it does not already exist
    # grpLis = hObj.keys()
    # if not "PFT" in grpLis:
    #   pftGrp = hObj.create_group("PFT")
    #else:
    #   pftGrp = hObj["PFT"]
    # TBD, review this, see if we still need it. 
    # baseNam = srcPath[srcPath.find("_M")+1:].replace('.uint8','')
    #
    datasetName = hDatasetName
    logMsg(logf,"@A createNewHdf5Dataset:: creating dataset [%s] type(%s) from %s"%(hDatasetName,dataType,srcPath))
    
    # Add this current dataset to the HDF5 file,hObj
    if nRows > 1:
        dtaArray = np.fromfile(srcPath,dtype=dataType).reshape(nRows,nCols)
        # print("2D SHAPE %d %d"%( dtaArray.shape[0],dtaArray.shape[1]))
        dsObj = hObj.create_dataset(datasetName,(nRows,nCols),dtype=dataType,chunks=True,data=dtaArray,compression='gzip',compression_opts=cmpLev)
    else:
        # print("1D %s SHAPE defined by user as [%d] for %s as dataype %s"%(datasetName, nCols,srcPath,dataType))
        dtaArray = np.fromfile(srcPath,dtype=dataType)
        # print("dtaArray new shape is %d filesize %d "%(dtaArray.shape[0],os.path.getsize(srcPath)))
        dsObj = hObj.create_dataset(datasetName,(nCols,),dtype=dataType,chunks=True,data=dtaArray,compression='gzip',compression_opts=cmpLev)
        # print("1D SHAPE %d defined by user as %d for %s"%(dtaArray.shape[0], nCols,srcPath))
    
    # also, Create INPUTPOINTER_ANC_xxx provenance attribute for this dataset
    inputPointerAttribName = ("INPUTPOINTER_ANC_%3.3d"%(ithInput))
    logMsg(logf,"@A adding provenance entry %s"%(srcPath))
    hObj.attrs[ inputPointerAttribName ] = srcPath
    
    # Create dataset level attributes
    dsObj.attrs['land_cover_classification' ] = "PFT Plant Function Type (PFT) from MODIS MCD Type-5"
    dsObj.attrs['units'     ] = "categorical classification codes 1..8"
    dsObj.attrs['compression_method'    ] = 'gzip'
    dsObj.attrs['compression_level '    ] = cmpLev
    
    dsObj.attrs['short_name']     = basenameData
    dsObj.attrs['long_name']      = basenameData
    dsObj.attrs['valid_min']      = 'TBD'
    dsObj.attrs['valid_max']      = 'TBD'
    dsObj.attrs['missing_value']  = 'TBD'
    dsObj.attrs['fill_value']     = 'TBD'

    dsObj.attrs['n_rows'    ] = nRows
    dsObj.attrs['n_cols'    ] = nCols
    dsObj.attrs['n_pixels'  ] = nRows * nCols
    
    if datasetName.find("12Q1")>= 0:
        dsObj.attrs['data_source'        ] = srcPath
        dsObj.attrs['data_source_label'  ] = "MODIS MCD12Q1"
        dsObj.attrs['data_provider'      ] = "Lucas Jones, Univ. Montana"
        # dsObj.attrs['data_provider_POC'  ] = "Joe Glassy, Univ. Montana"
        # PFT_VEG = ENF=0   EBF=1   DNF=2   DBF=3   Shrub=4 Grass=5 Cereal_crop=6   Broad_leaf_crop=7 
        add_PFT_Legend(dsObj)

    # Return open handle to new HDF5 file in case more needs
    return hObj
    # end::createNewHdf5Dataset()


# ---------------------------------------------------------------------
def read_store_MD5s(h5f,h5Varname,md5Path,logf):
    '''
    function: read_store_MD5s(h5f,md5Path,h5Varname,logf)
    purpose : Opens the MD5s csv file that contains MD5s for all raw 
              binary inputs, creates a string type HDF5 datatype
              and stores the MD5's CSV text file as a string table inside
              the Ancillary file whose handle is h5f
    inputs  : h5f -- an open HDF5 file for this new SMAP_L4_C_Ancillary.h5 file
              h5BPUT_Varname -- the SDS name for this internal MD5s stored instance.
              lutPath --the fully qualified path where the text MD5s live
              logf -- handle to open log file
    returns : Populated string table containing all MD5's for raw binary input layers
    notes   : Uses NUMPY style string table handling, wrapped by h5py
    with open(inputPath) as inf:
     lis=[ln.strip("\n\r") for ln in inf if not ln.startswith("#")]
    
    '''
    strTbl=""
    # Create a special variable length STRING datatype
    thisStrType= h5py.special_dtype(vlen=str)
    if h5Varname not in hObj:
        with open(md5Path) as inf:
            logMsg(logf,"@A in read_store_MD5: storing %s now in %s"%(md5Path,h5Varname))
            strTbl = np.array([ ln.strip("\n\r") for ln in inf], dtype=thisStrType)
            dset= h5f.create_dataset(h5Varname,(len(strTbl),),dtype=thisStrType,data=strTbl)
        # NOTE: Here we show the idiom to read this BACK into a list, just for example.
        # Convert the h5py dataset object, dset, from array into a simple list
        # strTbl=[dset[i] for i in range(len(dset))]
    else:
        logMsg(logf,"@E (%s) HDF5 variable name for MD5 is already defined -cannot overwrite!"%(h5Varname))
    # Returns string table containing the MD5s, just in case its needed
    return strTbl
    # end::read_store_MD5s()

# ---------------------------------------------------------------------
def read_store_JPG(h5f,h5Varname,imgPath,logf):
    '''
    EXPERIMENTAL: this is NOT working yet and is not complete@!
    function: read_store_JPG()
    purpose : Ingest a verbatim JPG image and store intact in an HDF5 dataset
    inputs  : h5f -- open HDF5 file handle
            : h5Varname -- name of variable to store
            : imgPath   -- full path to JPG image to store
            : logf      -- session log file handle (open)
    returns : dsObj -- a populated dataset    
    status  : Fails, on "Shape tuple is incompatible with data"    
    '''
    # for now, ignores h5Varname
    thisType= h5py.special_dtype(vlen=np.dtype('uint8'))
    fileSizeInBytes = os.path.getsize(imgPath)
    # imgData = np.fromfile(imgPath,dtype="uint8").flatten()
    with open(imgPath,'rb') as inf:
        rawBinData = inf.read()
        thisType = h5py.special_dtype(vlen=np.dtype('uint8'))
        dsObj = h5f.create_dataset(h5Varname,(fileSizeInBytes,), dtype=thisType,data=rawBinData)
    '''
    with open(imgPath,'rb') as inf:
        binary_data = inf.read()
        # create dataset containing jpeg binary data. Stores as a 1D
        # sequence of bytes, e.g. default type, here: string
        logMsg(logf,"@A in read_store_JPG: size(%d) storing %s now in %s"%(fileSizeInBytes,imgPath,h5Varname))
        dsObj = h5f.create_dataset(h5Varname,imgData.shape,dtype="uint8",data=imgData)
    '''
    return dsObj
    # end:: read_store_JPG()

    
# ---------------------------------------------------------------------
def read_store_BPLUT(h5f,h5Varname,lutPath,logf):
    '''
    function: read_store_BPLUT
    purpose : Opens the BPLUT csv file, creates a string type HDF5 datatype
              and stores the BPLUT CSV text file as a string table inside
              the Ancillary file whose handle is h5f
    inputs  : h5f -- an open HDF5 file for this new SMAP_L4_C_Ancillary.h5 file
              h5BPUT_Varname -- the SDS name for this internal BPLUT stored instance.
              lutPath --the fully qualified path where the text LUT lives
              logf -- open log file handle
    returns : Populated string table containing all BPLUT data              
    notes   : Uses NUMPY style string table handling, wrapped by h5py
    with open(inputPath) as inf:
     lis=[ln.strip("\n\r") for ln in inf if not ln.startswith("#")]
    
    '''
    strTbl=""
    # Create a special variable length STRING datatype
    thisStrType= h5py.special_dtype(vlen=str)
    if h5Varname not in hObj:
        with open(lutPath) as inf:
            strTbl = np.array([ ln.strip("\n\r") for ln in inf], dtype=thisStrType)
            logMsg(logf,"@A in read_store_BPLUT: storing %s now in %s"%(lutPath,h5Varname))
            dset= h5f.create_dataset(h5Varname,(len(strTbl),),dtype=thisStrType,data=strTbl)
            
        # NOTE: Here we show the idiom to read this BACK into a list, just for example.
        # Convert the h5py dataset object, dset, from array into a simple list
        # strTbl=[dset[i] for i in range(len(dset))]

    else:
        logMsg(logf,"@E (%s) HDF5 variable name is already defined -cannot overwrite!"%(h5Varname))
    # Returns string table containing the BPLUT, just in case its needed
    return strTbl
    # end::read_store_BPLUT()

# ---------------------------------------------------------------------
# main
# ---------------------------------------------------------------------
if __name__ == "__main__":
    '''
    main()
    '''
    # set operating parameters
    compressLev = 4  # boosted to 4, GZIP compression level 
    ancRootDir  = '' # TBD
    # create session log file
    dateLbl = datetime.datetime.now().isoformat()[0:10]
    masterLogPath = "l4c_genAncillary_" + VersId + "_"+ dateLbl + ".log"
    logf = open(masterLogPath,"wt")
    #
    LDOM1KM = 134787240
    LDOM3KM = 56359296
    LDOM9KM = 1664040
    #
    nRows = 14616
    nCols = 34704
    # Grid extents for various supported EASEGRID resolutions
    nRows1km = 14616
    nCols1km = 34704
    #
    nRows3km =  4872
    nCols3km = 11568
    #
    nRows9km = 1624
    nCols9km = 3856
    #
    doyLutPath = "doy_to_period_lut.csv"
    bplutPath  = "SMAP_BPLUT_Vv60XX_rev_20210908.csv"
    inputListPath       = "dataset-undefined"
    ancOutputHdf5Path   = "output-hdf5-undefined"
    dType = "datatype-undefined"
    #
    if len(sys.argv) >= 3:
        ancOutputHdf5Path = sys.argv[1]
        inputListPath  = sys.argv[2]
        # ...optional params
        if len(sys.argv) >= 5:
            bplutPath = sys.argv[3]   # both now mandatory
            md5sPath  = sys.argv[4]   # mandatory
            if len(sys.argv) >= 6:
                compressLev    = int(sys.argv[5])
                if len(sys.argv)>= 7:
                    doyLutPath = sys.argv[6]
    else:
        print("%s invocation: <AncillaryHdf5Path> <inputListPath> <doyLut.csv> {CompressLevel} "%(sys.argv[0]))
        sys.exit(-1)
        
    #
    strtSec = time.time()
    logMsg(logf,"@A %s started at %s"%(sys.argv[0],time.ctime()))
    logMsg(logf,"@A path to input files to ingest:  [%s]" % inputListPath)
    
    # Ingest raw list of Ancillary raw binary (dataset) images to load
    # <JMG> 2014-05-08, added ability to exclude comment lines
    if not os.path.exists(inputListPath):
        logMsg(logf,"@F Cannot find or open list of files to ingest into Ancillary file!"%(inputListPath))
        sys.exit(-1)
        
    with open(inputListPath) as inf:
        inpLis =[ln.strip('\n\r') for ln in inf if not ln.startswith('#')]
        for thisInput in inpLis:
            logMsg(logf,"CANDIDATE INGEST COMPONENT %s"%(thisInput))

    # Create new ancillary file SMAP_L4_C_Ancillary.h5
    #    each time this is run
    logMsg(logf,"@A creating NEW ancillary file %s"%(ancOutputHdf5Path))
    hObj = h5py.File(ancOutputHdf5Path,"w")
    # Create file level attributes (of type string)
    createFileAttributes(hObj,ancOutputHdf5Path,logf)

    #  Open, read, store a copy of the BPLUT inside this Ancillary file
    h5_bplut_varname = "TEXT_BPLUT"
    read_store_BPLUT(hObj,h5_bplut_varname,bplutPath,logf)
    #  Open, read, store a copy of the MD5's inside this Ancillary file
    h5_md5s_varname = "TEXT_MD5S"
    read_store_MD5s(hObj,h5_md5s_varname,md5sPath,logf)
    
    imgPath = "MOD12Q1_M09_lc_dom.uint8"
    h5Varname = os.path.splitext(os.path.basename(imgPath))[0].upper()
    '''
    if os.path.exists(imgPath):
        read_store_JPG(hObj,h5Varname,imgPath,logf)
    '''
    hObj.flush()

    # Create the new UMT_META group
    umtMetaGrp = hObj.create_group("UMT_META")
    
    # Create subgroup to store static RIP parameters in
    # and call routine to inject UM META specific parameters
    # into this group. Remember: fetch a reference to a group
    # by later referring to it this way: umMetaGrp = hObj["UMT_META"]
    createUM_META_Attributes(hObj)

    # the RIP metadata group is TBD -- not yet implemented
    # umtMetaRipGrp = hObj.create_group("UMT_META/RIP")

    #  Create DAY-OF-YEAR Lookup table
    # DEPRECATE, skip for now
    # create_doy_lut(hObj,doyLutPath)

    # Iterate through input file paths, ingesting each
    # each must have a datatpe qualifier like .flt32 as an extension
    ithInput = 0 
    for curPath in inpLis:
        ithInput += 1
        # must exist, and skip over the missing counts file
        if os.path.exists(curPath) and curPath.find("nmsng") < 0:
            # infer datatype based on extension like uint8...
            # pass in ithInput to populate INPUTPOINTER_ANC_iii
            createNewHdf5Dataset(hObj,curPath,compressLev,ithInput,logf)
        else:
            logMsg(logf,"@E Cannot find input raw binary file(%s)!"%(curPath))
            sys.exit(-1)
            # continue

    hObj.close()
    # Freshen the TOC, using stable htoc_v1.5.py utility
    cmd= "python ~/bin/htoc_v1.5.py " + ancOutputHdf5Path
    os.system(cmd)
    cmd= "ls -lt " + ancOutputHdf5Path
    os.system(cmd)
    
    endiSec = time.time()
    elapSec = endiSec - strtSec
    logMsg(logf,"@A task %s complete at %s requiring %.2f sec"%(sys.argv[0],time.ctime(),elapSec))
    logf.close()
    # sys.exit(0)
    # end::main()
