// ---------------------------------------------------------------
// module  : frm_log_msg.h
// purpose : implement "framework" component function prototypes for xLogMsg() and friends
// revised : v2.2-2015-10-14T17:06, introduced xLogOpenfile(path) with simpler
//           semantics to clarify confusion with xOpenLogfile(path,NULL) usage
// revised : v2.1-2014-02-09T15:52:00-07:00, made xLogMsg() safer, uses stdout if curr logf not open
// revised : v2.0-2014-01-21T16:52:00-07:00
// history : v2.0-2014-01-21T16:52:00-07:00, all unit tests passed
// history : 2014-01-20T15:28:00,madison,jmg: added xLocalLogMsg(logf,"msg"...) and USE_DEFAULT_LOG semantics
// history : 2014-01-05T21:02:00,madison,jmg, added LogMgr pre-prototype (not yet tested)
// history : 2014-01-03T13:42:00, corrected ability to now free IsStringTable
// author  : joe glassy, Lupine Logic Inc.
// notes   : n/a
// ---------------------------------------------------------------

#ifndef FRM_LOG_MGR_DEF
#define FRM_LOG_MGR_DEF 1

// include all common, core symbols for frm_api
#include "frm_core.h"


#define FRM_LOG_VERSTAG "v2.1-2014-02-09T15:52:00-07:00"

// helper: xOpenLog(USE_DEFAULT_LOG) ..and xCloseLog(USE_DEFAULT_LOG);
#define USE_DEFAULT_LOG   NULL


typedef struct tagLogObj {
 char logObjPath[FILENAME_MAX]; // POSIX filepath
 char logObjName[256] ; // text label for this instance
 FILE *logObjf ;    // pointer to POSIX (high level) file handle
 int  logObjOwner ; // unique ID of process or thread owning this log instance
 int  logObjState ; // {IsAvailable,IsNotAvailable}
 int  logObjLinesWritten; // count of traffic (n. lines) emitted to this log
} LogObj ;


// Prototype of thread-safe log manager. Each thread manages its
//    own independent LogObj.
typedef struct tagLogMgr {
  int logMgrMasterOwner; // ID (process or thread id) owning this LogMgr
  int nLogObjects  ; // count of active log objects
  LogObj *logObj_p ; // array of LogObj objects	
  char logMgrName[256]; // name of LogMgr itself, for memObjCalloc...  
} LogMgr ;

// ---------log_msg.c (from orig: l4c_logmsg.c) PROTOTYPES -------------

char *xLogGetVersTag(void);

// single-instance (shared) session log functions
time_t xOpenLogfile(char *logPath,FILE *logf);
// <JMG> 2015-10-14 plan to migrate all future use to this form below
// and migrate to use name xLogOpenfile()
time_t xLogOpenfile(char *logPath) ;

void xLogClose(FILE *logf);
// this will soon be deprecated to just use xLogClose()
void xCloseLog(FILE *logf);

void xLogMsg(char *fmtMsg_p,...);

// local version, where caller submits their own open file handle
void xLocalLogMsg(FILE *thisLogf,char *fmtMsg_p,...) ;

// DEPRECATING THIS FUNCTION SOON
FILE *xGetLogfileHandle(void);
// NEW, USE and migrate all usage to xLogGetFilehandle()
FILE *xLogGetFilehandle(void);

// re-assigns single active log stream, currentLogf handle..
void xSetCurrentLogHandle(FILE *usersLogf);

// thread-safe facility to create independent instance of logs
LogMgr *logMgrCreate(char *thisLogMgrName);
// logPathTable_p[] not yet fully implemented..
int logMgrInit(LogMgr *logMgr_p,const int nLogObj,char **logPathTable_p);

// --------------------------------------------------------------------
// LogMgr segment, "instance-replicable" log model, to transition to thread-safe usage
// where each thread owns its own log session as an instance of LogObj, managed by
// LogMgr.
void logMgrRelease(LogMgr *logMgr_p);

void   logMgrMessage(LogObj *logObj_p,char *fmtMsg_p,...);
LogObj *createLogObjInstance(LogMgr *logMgr_p,char *logPath);
void  releaseLogObjInstance(LogMgr *logMgr_p,LogMgr *logMgr);
LogObj *getLogObjInstance(LogMgr *logMgr_p,char *logName);

#endif


