// -----------------------------------------------------
// module       : frm_str_fx.h
// purpose      : Implement headers for the "framework" a set of C string functions 
//                missing from POSIX's string.h
//                to provide token-level parsing, scalar-to-list (to/from) functions.
// version      : v1.6-2017-04-26T18:54
// revision     : 2017-04-26, added frm_StrTrimWhitespace(sourceStr_p)
// revision     : 2014-0-13T11:17 added  *frm_strAddDatetimeStamp()
// history      : 2014-02-11T15:10, added missing frm_strToUpper(), frm_strToLower()
// history      : 2014-01-28T20:49:00, added frm_strGetTokenAsFloat64(), and
//                frm_strGetTokenAsNumeric() and frm_strGetTypedValue()
// history      : 2014-01-23T12:00:00-07:00
// history      : v1.3-2014-01-23T12:00:00,added 2 fx: 
//                 frm_strGetTextfileWidth() and
//                 frm_strReadTextIntoTable() ...reads a text file into a simple IsStringTable
// history      : 2014-01-21T17:10, added to  unit tests
// history      : 2014-01-14T17:18:00,renamed all "lli_" to "frm_" for framework consistency
// author       : joe glassy, Lupine Logic Inc.
// dependencies : tbd
// history      : 2014-01-007T15:38:00, orig consolidation of MUM fx into one file
// notes        : code originally in UM MODIS (MUM) API, adapted for L4_C framework context
// -----------------------------------------------------


#ifndef FRM_STR_FX_DEF
#define FRM_STR_FX_DEF   1

// include all common, core symbols for frm_api
#include "frm_core.h"


#define FRM_STR_APP_NAME "frm_str_fx.ex"

#define FRM_STR_VERS_TAG "v1.7-2017-04-27T11:02:00-07:00,jmg"


// re-certify the dependence on TIV
#ifdef HAVE_TIV
#include "frm_tiv_api.h"
#endif

#include "hdf.h"
#include "mfhdf.h"


// for: frm_strCustomFmt() supported operations codes:
enum CustomOps {
	DoConcatenate, DoPrepend 
} ;


// -----------------------------------------------------
// FUNCTION PROTOTYPES (namespace prefix: "frm_str"
// -----------------------------------------------------
char *frm_strGetVersTag(void);

char *frm_strToUpper(char *text_p);
char *frm_strToLowerr(char *text_p);

void *frm_strToTypedValue(void *varBuf,char *valueStr,const int tivDatatype);

void *frm_strGetTokenAsNumeric(void *resultValBuffer,const int tivDatatype,
		char *sourceStr,const int ithToken,const char *tokenSepChars);

float64 frm_strGetTokenAsFloat64(void *resultValu,char *sourceStr,
	const int ithToken,const char *tokenSepChars);

int   frm_strChrCount(const char *text,const int targetChr) ;
char *frm_strCpBetween(const char *text,char *resultString,int startTok,int endTok,int *resultLength);
int   frm_strChrIndex(const char *text,const int targetCh);
char *frm_strRemoveChr(char *text,const int charToRemove);
char *frm_strChrRemove(char *text,const int charToRemove);
int   frm_strCountTokens(char *sourceStr,const char *tokenSepChars);

// <JMG> 2017-04-26T18:53:00 added, to allow RIP parsing to be more robust
// remember you MUST fetch modified result from function return!!
char *frm_StrTrimWhitespace(char *sourceString_p) ;

// With this (buffer) form, its ok to just call and retrieve dest_p
char *frm_StrTrimWhitespaceToBuffer(char *str_p,char *dest_p);

char *frm_strGetToken(char *resultStr,char *sourceStr,const int ithToken,const char *tokenSepChars);

char *frm_strXtracRoot(char *source,char *dest,const int sep_char);
char *frm_strXtracExt(char *source,char *dest,const int sep_char);
char *frm_strPrepend(char *dest_p, char *source_p, const char *prepend_str_p);
int   frm_strSubstringIndex(char *srcStr_p,char *targStr_p);
char *frm_strReplaceChr(const char *targetChSet_p,char *srcAndResult_p,const char *newDest_char);
int   frm_strParseValueList(char *source_p,char *delimStr,const int datatypeCode,void *resultArray_p);
int   frm_strLoadValueStringFromArray(char *resultStr_p,const char *fmtSpec,const int delimCh,const int nValues,const int tivDatatype,void *resultArray_p);
char *frm_strCustomFmt(const int op_code,char *origBuffer_p,
                     char *fmtSpecStr_p,...);

// <JMG> added 2014-0-13T11:17
char *frm_strAddDatetimeStamp(const time_t finalSec,const int whichTimezone,char *prefixPath,char *localTzLabel,char *finalFilepath);

// <JMG> added 2014-07-29T19:36:00-0700
char **frm_strExpandStringTbl(const int nOrigStrings,const int expandByCh,char **origStrTbl_p);

// TEXT FILE level processors
int  frm_strGetTextfileWidth(const char *inputTextPath_p,int *lineCount);
char **frm_strReadTextfileIntoStringTable(const char *inputTextPath_p,int *linesProcessed);

char **frmReadIntoStringTbl(char *inputPath_p,int *nLines);
size_t frm_IoGetFilesize(const char *inputPath_p);

#endif
