// -----------------------------------------------------
// module       : frm_util_fx.h
// purpose      : Implement headers for generic utility functions
// revised      : 2014-11-03T16:40, added frm_utilGetHostnameAndUser()
// history      : 2014-10-21T10:27:00, added frm_utilDiagImage() and
//                frm_utilGenFreqDistrib() adapted from L4CMDL l4c_ family
// history      : 2014-08-21T13:22, added FrmApiUniObj, and
//                 frm_utilCalcUpdates() and frm_utilCalcUnivariates()
// history      : <JMG> 2015-08-12T10:33:00-07:00, added, as work-around when NCCS' libuuid.a
//                not available or difficult to link into
// history      : 2014-07-21, added frm_utilAssignBits() missing earlier
//                to support insertion of integer (multiple-bit) values
//                into a bitflag integer at a range of bit positions.
// history      : 2014-06-13T16:28:00, added frm_utilExecOsCmd() service
//                function
// history      : v1.2-2014-02-09T16:34:00,  moved file related fx to frm_io_fx.c
// history      : 2014-01-28T11:37:00-07:00, orig re-factor from MUM API
// history      : n/a
// author       : joe glassy, Lupine Logic Inc.
// dependencies : tbd
// history      : 2014-01-28T11:37:00, deprecated 2nd text file parser and now just
//                support one version reading whole text into raw binary buffer, 
//                named : frm_utilReadTextIntoTable()
// history      : 2014-01-24T15:38:00, orig consolidation of MUM fx into one file
// notes        : Code originally in UM MODIS (MUM) API, adapted for L4_C framework context
// -----------------------------------------------------


#ifndef FRM_UTIL_DEF
#define FRM_UTIL_DEF   1


// include all common, core symbols for frm_api
#include "frm_api.h"


#define FRM_UTIL_APP_NAME "frm_util_fx.ex"

// tag updated only for major milestones, not checkpoints
#define FRM_UTIL_VERS_TAG "v1.5-2014-11-03T16:40:00-07:00"

// <JMG> 2014-09-12T12:36
typedef struct tagGenericMetrics {
 float64 obs_min ;  // observed minimum (non-fillvalue)
 float64 obs_max ;  // observed maximum (non-fillvalue)
 float64 obs_range;  // observed range   (non-fillvalue)
 float64 obs_mean;  // observed mean   (non-fillvalue)
 float64 obs_stddev;// observed std deviation (non-fillvalue)
 float64 obs_stderror;    // observed std error (non-fillvalue)
 float64 obs_pctstderror; // observed percent std error (non-fillvalue)
 float64 obs_coef_var   ; // observed percent std error (non-fillvalue)
 float64 optionScaleFactor; // optional 'scale-by' factor to transform prior to output
 // Below are useful when a valid_min,valid_max bracket is available
 float64 minAllow ; // optional, value of smallest qualifying pixel value
 float64 maxAllow ; // optional, value of largest qualifying pixel value
 size_t  obs_CountQualified; // count of non-fillvalue pixels
} GenericMetrics ;


// adapted from MUM API's UNI.. changed counts to size_t...
typedef struct tagFrmApiUniObj {
  int     ulim_init ; // flag 0=when not init, or 1=when init
  uint8   ignore_background;  //  boolean,isTrue as needed               
  // uint8   select_parm[MAX_UNIVAR_PARAMS] ..this is a future boolean rpt list         
  int32   set_id ;     // unique ID of this univar set                  
  int32   uclass_id;   //  base-1 maskclass (KEEP! not same as Tclass!) 
  int32   rpt_form;    // {rptFormatted,rptListWise,rptParamWise,rptImageWise}
  size_t  n_samples;   // data sets sample size, N                   
  size_t  n_uf_bins;   // count of uf_bins allocated (0 if not used) 
  size_t  uf_invalid;  // count of under or over runs to unit-bin updates 
  size_t  n_ignored;   // n. (background) observations filtered out  
  float64 ulim_min;    // <JMG> added for FRM API adaptation
  float64 ulim_max;    // <JMG> added for FRM API adaptation
  float64 range  ;     // numeric value range, as (max-min)          
  float64 minority;    // least frequently occurring value           
  float64 majority;    // mode, most frequently occuring             
  float64 median ;     // median value in distribution               
  float64 sum    ;     // simple arithmatic sum                      
  float64 sum_sq ;     // sum of squares                             
  float64 avg    ;     // simple arithmatic mean                     
  float64 stddev ;     // sample standard deviation                  
  float64 std_err;     // standard error                             
  float64 pctserr;     // percent standard error                     
  float64 coef_var;    // coefficient of variation                   
  float64 background;  // background value, to optionally ignore     
  float32 uf_binwidth; // factor for binwidth,default is 1.0         
  int32  *uf_bins ;  //  ptr to optional unit-bins for freq             
} FrmApiUniObj;


// -----------------------------------------------------
// FUNCTION PROTOTYPES
// -----------------------------------------------------

// <JMG> 2014-09-12, added, returns {0|1}, from input choices:
// {"YES","ON","TRUE","ENABLED","1"}|{"NO","OFF","FALSE","DISABLED","0"}
int frm_utilTestBooleanStr(const char *optionBooleanSwitch_p);

// any '_Fillvalue' MUST lie outside of valid min,max here..
float32 frm_utilSummarizePixels(GenericMetrics *postMetrics_p,
        const char *dsetName_p,const hid_t hdf5Id,
        const int doWriteMetrics,const size_t pixelCount,float32 *sciAry_p);        

// <JMG> 2014-07-31, added orig
double frm_utilEventCompletionMsg(const char *thisFunction_p,const time_t startSec,
        const int enableCompletionMsg) ;

// <JMG> 2014-06-13 added orig
int frm_utilExecOsCmd(char *fmtSpecStr_p,...);

pid_t        frm_utilGetPid(pid_t *parent_pid) ;
char         *frm_utilGetCtime(char *timBuf,const time_t usersTime);

// evaluate the platform architecture's byte-ordering as little or big endian
// returning: 
uint8    frm_utilTestByteorder(void);
char     *frm_utilDecToBinarystr(uint32 num,const int32 width,char *result_str);

// assign one bit at a time into a bitflag integer
uint32       frm_utilSetBits(const int16 startPos,const int16 endPos, void *data, size_t dataSize);

// assign a multiple-bit integer into a larger bitflag integer using AssignBits
uint32       frm_utilAssignBits(const int16 startPos, const int16 endPos, void *destData, size_t dataSize,
                      void *newValueMask, size_t maskSize);
                     
// uses mumGetBits() style logic to retrieve a range of 1 or more bits from a bitflag integer
uint32       frm_utilRetrieveBits(const int16 startPos,const int16 endPos, void *data,size_t dataSize);

// uses K&R logic to fetch a range of bits from a integer up to a uint32. 
// When using be sure to explicitly case input int to uint32
uint32       frm_utilGetBits(const int32 strt_at,const int32 end_at,uint32 sourceData);


// UUID info...
//             0 --> displays real UUID as a  packed string
//             1 --> displays real UUID as a normal hypenated couplet form of output, as from uuid_unparse()
//             2 --> forces generation of a pseudo UUID and displays it in hyphenated couplet form
//             3 --> forces generation of a pseudo UUID and displays it as a packed string
//           or if displayMode==2, produces a dummy random placeholder (not authentic) UUID
char *frm_utilGenerateUUID(char *resultUUID_String,const int displayMode);
// <JMG> 2015-08-12T10:33:00-07:00, added, as work-around when NCCS' libuuid.a
// not available or difficult to link into
char *frm_utilPrivateUUID(char *resultPrivateUUID_Str);

int  frm_utilGetISO8601_datetime(time_t usersSec,char *loc_dtStr_p,char *utc_dtStr_p);

int    frm_utilCalcFrequencies(char *taskLabel_p,uint8 *sourceImg_p,const int32 nElem,FILE *destf);
size_t frm_utilCalcUpdates(FrmApiUniObj *uni_p,const double value);
double frm_utilCalcUnivariates(FrmApiUniObj *uni_p);

// <JMG> 2014-11-05T16:44:00, added to allow use of biased vs. unbiased and
// corrected vs. uncorrected methods as needed
double frm_utilCalc_std_dev_and_var(const size_t vCount,const double vSum,
        const double vSumSquares,double *vVariance_p,int *errStatus);

// <JMG> 2014-10-21T10:22:00-07:00, added support for on-the-fly image diagnostics using
// external dependency genDiagImg.py when available
int frm_utilDiagImage(char *imgPrefix, char *outputPath,int tivDatatype,
            size_t nPixels, size_t *dims,void *imgData_p);
// <JMG> 2014-10-21T10:22, added, currently limited to uint8 images, will
// extend to other datatypes in future
int frm_utilGenFreqDistrib(char *imgPrefix, char *outputPath,int tivDatatype,
            size_t nPixels, void *imgArray_p) ;

#endif

