# ---------------------------------------------------------------------
# file     : frm_api.mak
# purpose  : Makefile for frm_api, to build the static library, L4_C FRAMEWORK (libfrm.a or later, libfrm.so)
# dependencies : env vars FRM_API_INC and FRM_API_LIB pointing to canonical directories
#             where 
# revised  : v1.5.8-2015-07-10T12:06, verify gcc 4.8.x use, no code changes otherwise.
# revised  : v1.5.7-2014-02-21T00:37:00, added frm_hdf_io.h into ./inc, and frm_hdf_io.c into
#                ./src to bring into main API Makefile build process. frm_hdf_io.h only includes
#                minimal functionality (open,close,read,write) until I can get more time
#                to bring the rest in. also added pkgMgr_GlobalReport()
#
# revised  : v1.5.4-2014-02-21T08:53:00, added 
# revised  : v1.5.3-2014-02-16T08:53:00, added nsb into Framework, added nsbKeyPolicy switch.
# revised  : v1.5.2-2014-02-12T21:44:00-07:00 integrated pkg_mgr (client) into main FRM API build
# revised  : v1.5.1-2014-02-10T16:08:00-07:00 added make test to force test of link+run with newest libfrm.a
# revised  : v1.4-2014-02-01T10:10:00, added frm_core.c, for frm_GetApiVersion(versStr);
#            also note that devel of experimental gAryAdt started, but not yet implemented in API.
# history  : v1.3-2014-01-30T13:24:00, added -DHAVE_PTHRD for those platforms that have it
#            and changed all prefix R_ to FRM_API_ for all xxx_RANDOM macros.
# history  : 2014-01-24T18:08:00, migrated to model with all sources stored in ./src, was ./api_src and
#            all headers in ./inc
# history  : 2014-01-23T11:08:00, added single header frm_lib.h for library libfrm.a which
#            just collects all components into a single library for convenience.
# history  : 2014-01-21, added more unit tests
# history  : 2014-01-21, for unit test
# author   : joe glassy
# platform : all on NTSG NIS for now.  
# notes    : Ideally, a single copy of each component API's headers should be placed only in 
#            ./Framework/api/inc directory

NEWBASE_GSFC=/discover/nobackup/projects/gmao/share/dao_ops/Baselibs/v4.0.3_build1/x86_64-unknown-linux-gnu/ifort_13.1.3.192-mvapich2_1.9/Linux
OLDBASE=/discover/nobackup/projects/gmao/share/dao_ops/Baselibs/v3.2.0_buildSLES11/Linux

# for Lupine Logic Inc Wave & Circuit on CentOS 7.7 host 'missouri' implementation
# NEWBASE=/missouri/stage/L4CSYS/L4CAPI/src
NEWBASE=/usr
BASE=$(NEWBASE)

HDF4INC=$(BASE)/include/hdf
HDF4LIB=$(BASE)/lib
# adapted for LLI host missouri
HDF5INC=$(BASE)/include
HDF5LIB=$(BASE)/lib64
SZLIBINC=$(BASE)/include/szlib
SZLIBLIB=$(BASE)/lib64

FRM_API_ROOT=/missouri/sw/L4CSYS/L4CAPI
FRM_API_SRC = $(FRM_API_ROOT)/src
# in case these aren't explicitly defined we define these here:
FRM_API_INC = $(FRM_API_ROOT)/inc
FRM_API_LIB = $(FRM_API_ROOT)/lib
#
# probably obsolete FRMLIB = ../lib
FRMLIB = $(FRM_API_LIB)

# CC = icc
# CC = gcc
# Verify use of new v4.8.x gcc compiler
# CC = /opt/rh/devtoolset-2/root/usr/bin/gcc
CC = gcc

CFLAGS =  -g -Wextra -Wall -std=c99 -fPIC  

ADDFLAGS = -D_GNU_SOURCE -DHAVE_UUID -DHAVE_HDF4 -DHAVE_PTHRD

# the -t option enables verbose TRACE of the link actions
# LDFLAGS = -t -ldf -lmfhdf -lhdfeos -lGctp -lhdf5 -lhdf5_hl  -lz -luuid -lm

# note that duma is not available on wheat or on all other platforms
# Replaced all use of Electric Fence with newer supported 'duma'

# note that -lz and -lsz are NOT yet technically needed by frm_api 

LDFLAGS = -ldf -lmfhdf -lhdf5 -lhdf5_hl -luuid -lpthread -lz -lsz -lm

# added...
ARTOOL  = ar
ARFLAGS = rv
# added...
.SUFFIXES : .o .c .a .h

INCDIR =.

# removed HDFEOS_INC/LIB dependencies for now
INCS = -I$(INCDIR) -I$(HDF4INC) -I$(HDF5INC) -I$(FRM_API_INC) -I$(SZLIBINC) -I.

# include -L. if libuuid.a is local here
LIBS = -L$(HDF4LIB) -L$(HDF5LIB) -L$(SZLIBLIB) -L. -L/usr/local/lib 

TARGET=libfrm.a

#  First list all core modules...then API clients like pkg_mgr, nsb_api etc
SRCS = frm_core.c \
	frm_mem_obj.c \
	frm_log_msg.c \
	frm_str_fx.c  \
	frm_tiv_api.c \
	frm_io_fx.c \
	frm_util_fx.c \
	frm_hdf_io.c\
	nsb_api.c \
	nsb_dbsys.c\
	nsb_accessors.c \
	nsb_parsers.c \
	pkg_mgr_objects.c \
	pkg_mgr_themes.c \
	pkg_mgr.c 


#Create one object file per .c source file
OBJS = $(SRCS:.c=.o)

# Update the .o files whenever a newer .c file is encountered
.c.o :
	$(CC) $(CFLAGS) $(ADDFLAGS) $(INCS) -c $< 

# Rule to update the .a library whenever a newer .c file is encountered
.c.a :
	$(CC) $(CFLAGS) $(ADDFLAGS) $(INCS) $<
	$(ARTOOL) $(ARFLAGS) $@ $*.o
	rm -f $*.o

# Rule to update the .a library whenever a newer .o file is found
.o.a :
	$(ARTOOL) $(ARFLAGS) $<

# from each .o combine to construct the static library, .a
$(TARGET): ${OBJS}
	$(ARTOOL) $(ARFLAGS) $@ $?
	
install :
	cp -p ${TARGET} $(FRMLIB)
	# echo ${TARGET} " built on " `date` " API Version " `grep VERS_TAG ../inc/frm_core.h | cut -d ' ' -f 3`
	echo ${TARGET} " built on " `date` " API Version " `grep VERS_TAG ../inc/frm_core.h`
	rm -f *.o *.a 
	ls -lt ../lib

clean :
	rm -f *.o *.a *.ex ../lib/*.a  2> /dev/null

all : clean ${TARGET} install test 

update :  ${TARGET} install

test :
	make ../frm_tiv_api
 
# done

