# ----------------------------------------------------------------------
# file         : l4c_MODIS_LLI.mak
# purpose      : Makefile for l4c_MODIS executable
# revised      : 2021-07-14T16:24, added references to BOTH hdfeos2 and hdfeos5
#                at same time.
# revised      : 2017-02-09T16:11, added l4c_MODIS_FparQaBin.[ch], as these are
#                now required for MOD15A2H processing.
# revised      : 2014-10-14T12:45:00-07:00, substituted l4c_MODIS_WriteMetadata.c
#                for prior l4c_MODIS_metadata.c, and note that l4c_MODIS_bplut.c 
#                is built, linked, but is NOT yet used
# revised      : 2014-05-01T14:59:00, stubbed in l4c_MODIS_metadata.c
# history      : 2021-07-14T16:24, added hdfeos2 and hdfeos5 references at same time.
# history      : 2013-11-14T13:20:00-07:00,NCCS for D5.0, jmg
# author       : joe glassy
# platform     : stout
# dependencies : libuuid.a, HDF5 v1.8.x, HDF4 v4.x, HDFEOS v2.x libraries
# notes        : n/a

CC = gcc

# for now, removed -DHAVE_UUID 
# NEW LDFLAGS -lhdfeos -lGctp -lmfhdf -ldf -lhdf5_hl -lhdf5 -lz -lsz -ljpeg -lm
# NEW CFLAGS :  -Wall -Wextra -g -t -fPIC 
# CFLAGS =  -g -Wall -Wextra -std=c99 -fPIC -D_GNU_SOURCE  -DHAVE_UUID

# CFLAGS =  -g -Wall -Wextra -fPIC  -DHAVE_UUID
CFLAGS =  -g -Wall -Wextra -fPIC -D_GNU_SOURCE -DHAVE_UUID -DHAVE_HDF4 -DHAVE_HDF5

# temporary hard code references for LLI host missouri test, allowing both
# HDFEOS 2.18 (legacy MODIS HDF,HDFEOSv2) and newer HDFEOS 5.x calls at one time.
HDFEOS_ROOT=/usr/local/hdfeos_v2.18
HDFEOS_LIB=$(HDFEOS_ROOT)/lib
HDFEOS_INC=$(HDFEOS_ROOT)/include

# /usr/local/hdfeos5/lib/linux64
HDFEOS5_ROOT=/usr/local/hdfeos5
HDFEOS5_LIB=$(HDFEOS5_ROOT)/lib/linux64
HDFEOS5_INC=$(HDFEOS5_ROOT)/include

# NOTE: syntax that WORKS FINE on wheat is:
#  LDFLAGS =  -ldf -lmfhdf -lhdfeos -lGctp -lhdf5 -lhdf5_hl  -lz -luuid -lm
#
# Still developing a syntax that works on NCCS below:
# temp remove -Gctp and -hdfeos
# NOTE: this was used successfully at NCCS for D5.0
# LDFLAGS = -t -lhdfeos -lGctp -lmfhdf -ldf -lhdf5_hl -lhdf5 -lz -lsz -ljpeg -luuid -lm
#  save this prior , for laster restore if needed
# LDFLAGS = -lfrm -lhdfeos -lGctp -lmfhdf -ldf -lhdf5_hl -lhdf5 -lz -lsz -ljpeg  -luuid -lm
LDFLAGS = -lfrm -lhdfeos -lhe5_hdfeos -lGctp -lmfhdf -ldf -lhdf5_hl -lhdf5 -lz -lsz -ljpeg -luuid -lm

INCS = -I$(FRM_API_INC) -I$(HDFEOS_INC) -I$(HDFEOS5_INC) -I$(HDF4_INC) -I$(HDF5_INC) -I. 

# trying different order combinations...
# LIBS =  -L$(FRM_API_LIB)  -L$(HDFEOS_LIB) -L$(HDF4_LIB) -L$(HDF5_LIB)
# <JMG> 2020-12-17
LIBS = -L$(HDF5_LIB) -L$(FRM_API_LIB) -L$(HDFEOS_LIB) -L$(HDFEOS5_LIB) -L$(HDF4_LIB)
# LIBS = -L. -L$(ALL_LIB)  

TARGET = l4c_MODIS.ex

SRCS = 	l4c_MODIS_rip.c \
	l4c_MODIS_FparQaBin.c \
	l4c_MODIS_bplut.c \
	l4c_MODIS_ses.c \
	l4c_MODIS_PkgMgrInit.c \
	l4c_MODIS_fx.c \
	l4c_MODIS_io.c \
	l4c_MODIS_easegrid.c \
	l4c_MODIS_logic.c \
	l4c_MODIS_drv.c \
	l4c_MODIS_WriteMetadata.c \
	l4c_MODIS.c 

#Create one object file per .c source file
OBJS = $(SRCS:.c=.o)

all: $(TARGET)

# re-arranged order: the order in the CC line is very important to correct linking...
# note: LIBS refers to the -L locations, not the libraries themselves
# Prior:  $(CC) $(CFLAGS) -o $(TARGET) $(OBJS) $(INCS) $(LDFLAGS) $(LIBS)
$(TARGET): $(OBJS)
	$(CC) $(CFLAGS) -o $(TARGET) $(OBJS)  $(INCS) $(LIBS) $(LDFLAGS) 

test:
	./l4c_MODIS.ex l4c_MODIS.rip

clean:
	rm -f *.o *~ core $(INCDIR)*~ 

install:
	mv $(TARGET) ..

#Update the .o files whenever a newer .c file is encountered
.c.o:
	$(CC) $(CFLAGS) $(INCS) -c $< -o $@

