# ----------------------------------------------------------------------
# file         : l4c_MODIS.mak
# purpose      : Makefile for l4c_MODIS executable
# revised      : 2014-10-14T12:45:00-07:00, substituted l4c_MODIS_WriteMetadata.c
#                for prior l4c_MODIS_metadata.c, and note that l4c_MODIS_bplut.c 
#                is built, linked, but is NOT yet used
# revised      : 2014-05-01T14:59:00, stubbed in l4c_MODIS_metadata.c
# history      : 2013-11-14T13:20:00-07:00,NCCS for D5.0, jmg
# author       : joe glassy
# platform     : stout
# dependencies : libuuid.a, HDF5 v1.8.x, HDF4 v4.x, HDFEOS v2.x libraries
# notes        : n/a

# NOTE: This HDF4 works fine as a stand-alone HDF4 without HDFEOS but it does NOT
# seem compatible with the HDFEOS library I'm using below...
#   HDF4INC = /usr/local/other/hdf4/4.2.6/gcc-4.1.2/include/hdf
#   HDF4LIB = /usr/local/other/hdf4/4.2.6/gcc-4.1.2/lib

# NOTE: these are all static libraries
#  HDF4INC =/usr/local/other/hdf/4.2.5/include
#  HDF4LIB =/usr/local/other/hdf/4.2.5/lib 
# HDF4INC   = /discover/nobackup/jglassy/L4_C/include/h4
# HDF4LIB   = /discover/nobackup/jglassy/L4_C/lib/h4
# HDF5LIB   = /discover/nobackup/jglassy/L4_C/lib/h5
# HDF5INC   = /discover/nobackup/jglassy/L4_C/include/h5
# HDFEOS_LIB= /usr/local/other/hdfeos/2.16v1.00_GNU/lib
# HDFEOS_INC= /usr/local/other/hdfeos/2.16v1.00_GNU/include
# 
# SZINC=/discover/nobackup/projects/gmao/share/dao_ops/Baselibs/v3.2.0_buildSLES11SP1_IMPI4/Linux/include/szlib
# LIBZINC=/discover/nobackup/projects/gmao/share/dao_ops/Baselibs/v3.2.0_buildSLES11SP1_IMPI4/Linux/include/zlib
# HDFEOSINC=/discover/nobackup/projects/gmao/share/dao_ops/Baselibs/v3.2.0_buildSLES11SP1_IMPI4/Linux/include/hdfeos
# HDF5INC=/discover/nobackup/projects/gmao/share/dao_ops/Baselibs/v3.2.0_buildSLES11SP1_IMPI4/Linux/include/hdf5
# HDF4INC=/discover/nobackup/projects/gmao/share/dao_ops/Baselibs/v3.2.0_buildSLES11SP1_IMPI4/Linux/include/hdf
# ALL_LIB=/discover/nobackup/projects/gmao/share/dao_ops/Baselibs/v3.2.0_buildSLES11SP1_IMPI4/Linux/lib

CC = gcc

# for now, removed -DHAVE_UUID 
# NEW LDFLAGS -lhdfeos -lGctp -lmfhdf -ldf -lhdf5_hl -lhdf5 -lz -lsz -ljpeg -lm
# NEW CFLAGS :  -Wall -Wextra -g -t -fPIC 
# CFLAGS =  -g -Wall -Wextra -std=c99 -fPIC -D_GNU_SOURCE  -DHAVE_UUID

# CFLAGS =  -g -Wall -Wextra -fPIC  -DHAVE_UUID
CFLAGS =  -g -Wall -Wextra -fPIC -D_GNU_SOURCE -DHAVE_UUID -DHAVE_HDF4 -DHAVE_HDF5

# for D5.0.1 and up
FRM_API_LIB=$(FRM_API_BASE)/lib
FRM_API_INC=$(FRM_API_BASE)/inc

# re-confirmed HDFEOS binaries present on stout.ntsg.umt.edu on 05/01/2014
HDFEOS_ROOT=/usr/local/hdfeos_2.18
HDFEOS_LIB=$(HDFEOS_ROOT)/lib
HDFEOS_INC=$(HDFEOS_ROOT)/include

# NOTE: syntax that WORKS FINE on wheat is:
#  LDFLAGS =  -ldf -lmfhdf -lhdfeos -lGctp -lhdf5 -lhdf5_hl  -lz -luuid -lm
#
# Still developing a syntax that works on NCCS below:
# temp remove -Gctp and -hdfeos
# NOTE: this was used successfully at NCCS for D5.0
# LDFLAGS = -t -lhdfeos -lGctp -lmfhdf -ldf -lhdf5_hl -lhdf5 -lz -lsz -ljpeg -luuid -lm
#
LDFLAGS = -lfrm -lhdfeos -lGctp -lmfhdf -ldf -lhdf5_hl -lhdf5 -lz -lsz -ljpeg  -luuid -lm

INCS = -I$(FRM_API_INC) -I$(HDFEOS_INC) -I$(HDF4INC) -I$(HDF5INC) -I. 

# trying different order combinations...
LIBS =  -L$(FRM_API_LIB)  -L$(HDFEOS_LIB) -L$(HDF4LIB) -L$(HDF5LIB)
# LIBS = -L. -L$(ALL_LIB)  

TARGET = l4c_MODIS.ex

SRCS = 	l4c_MODIS_rip.c \
	l4c_MODIS_bplut.c \
	l4c_MODIS_ses.c \
	l4c_MODIS_PkgMgrInit.c \
	l4c_MODIS_fx.c \
	l4c_MODIS_io.c \
	l4c_MODIS_easegrid.c \
	l4c_MODIS_logic.c \
	l4c_MODIS_drv.c \
	l4c_MODIS_WriteMetadata.c \
	l4c_MODIS.c 

#Create one object file per .c source file
OBJS = $(SRCS:.c=.o)

all: $(TARGET)

# re-arranged order: the order in the CC line is very important to correct linking...
# note: LIBS refers to the -L locations, not the libraries themselves
# Prior:  $(CC) $(CFLAGS) -o $(TARGET) $(OBJS) $(INCS) $(LDFLAGS) $(LIBS)
$(TARGET): $(OBJS)
	$(CC) $(CFLAGS) -o $(TARGET) $(OBJS)  $(INCS) $(LIBS) $(LDFLAGS) 

test:
	./l4c_MODIS.ex l4c_MODIS.rip

clean:
	rm -f *.o *~ core $(INCDIR)*~ 

install:
	cp $(TARGET) ..

#Update the .o files whenever a newer .c file is encountered
.c.o:
	$(CC) $(CFLAGS) $(INCS) -c $< -o $@

