// ---------------------------------------------------------------
//  subsystem    : SMAP L4_C
//  module      : l4c_MODIS_prot.h
//  purpose     : header containing function prototypes
//  revision    : 2021-10-05T15:39
//  revision    : 2021-07-14T17:33, added l4c_read_hdfeos5() vs l4c_read_hdfeos2()
//                since VIIRs requires HDFEOS 5 and not v2
//  revision    : 2017-02-10T17:05, added prototypes for MOD15A2H functions
//                such as resample_tile_500m_to_1km()
//  revision    : 2014-08-19, added int l4c_readAndWriteLatLon_9kmCoordVars(void);
//  revision    : 2014-07-28T23:58:00
//  history     : 2021-10-05, added reorg_VIIRS_Qc() for VIIRS Qc support
//  history     : 2013-11-04T10:40:00-07:00, v4.1.3
//  version     : v5.0.1
//  invocation  : n/a
//  author              : Lucas Jones
//  history             : renamed gridfpar.h to l4c_MODIS.h to better conform to production naming protocols
//  notes               : in structure CONT, taskIdStr[] was missing, Joe added it to allow make process to complete.
//  Notes       : rescaled and type-converted all tile_xxx template image files
//  L4_C integration: Joe Glassy
// 

// ---------------------------------------------------------------

#ifndef l4c_MODIS_Prot_DEF
#define l4c_MODIS_Prot_DEF 1

#include "l4c_MODIS.h"
#include "nsb_api_prot.h"
#include "pkg_mgr_prot.h"

// <JMG> 2017-02-09T16:10 added and renamed *[ch],
// from .laj/smap/code/gridfpar/src as these
// were missing from the L4CMOD code set, and are now required
#include "l4c_MODIS_FparQaBin.h"

// ---------------------------------------------------------------
// FUNCTION PROTOTYPES
// ---------------------------------------------------------------

// --module: l4c_MODIS_rip.c -------------------------------------------
void  l4c_SesCollectCoreNvps(SesCtl *sesCtl_p);
void  l4c_SesCollectNvps(SesCtl *sesCtl_p);
void  l4c_ApplyRootDirToPaths(SesCtl *sesCtl_p);

// <JMG> 2014-10-14T15:25, adopted from L4CMDL, for future FparClimatology processing
// noting that the FparClimatology period wise LUT is also now included in l4c_MODIS_rip.c
int l4c_SetFparClimatologyDatasetName(const int compositePeriodIdx,char *lutPath_p,char *dsetName_p);

// --module: l4c_MODIS_ses.c -------------------------------------------
L4C_ISO_MetaElements *l4c_GetISO_MetaElements(void) ;
SesCtl  *l4c_GetSesCtl(void);
NsbObj  *l4c_GetNsbOverseer(void) ;
PkgMgr  *l4c_GetPkgMgrOverseer(void) ;
ImgPkg  *l4c_GetImagePackage(void) ;
time_t  l4c_InitSession(int argc,char *argv[]);
void    l4c_CloseSession(SesCtl *sesCtl_p);

double l4c_EventCompletionMsg(const char *thisFunction_p);
double l4c_EventHighWaterMarkMsg(const char *thisFunction_p);

control_struct *l4c_GetLegacyCtl(void) ;
cont_struct    *l4c_Get_cont_struct(void) ;

// borrowed from l4c_DAILY etc
char *l4c_Ses_LogLabelTime(const time_t finalSec,char *finalLogPath);
int  l4c_Ses_DatestampLog(SesCtl *sesCtl_p);
// <JMG> 2014-10-14, updated to output ending with S.000Z UTC form
int l4c_GetISO8601_datetime(time_t usersSec,char *loc_dtStr_p,char *utc_dtStr_p);

// <JMG> added 2014-10-14, to support instrumented diags later
char *l4c_TIVDatatypeToDatatypeString(const int TIV_datatypeId,char *typBuf);
int l4c_DiagImage(char *imgPrefix, char *outputPath,int tivDatatype,size_t nPixels, size_t *dims,void *imgData_p);
int l4c_GenFreqDistrib(char *imgPrefix, char *outputPath,int tivDatatype,size_t nPixels, void *imgArray_p);


// ----------- l4c_MODIS_PkgMgrInit.c legacy model prototypes -------
// borrowed from l4c_MET...
int l4c_PkgMgr_Init(void);
void l4c_PkgInit_SMAP_L4_C_Ancillary(PkgTheme *pkgTheme_p,char *themeLabel_p);
void l4c_PkgInit_SMAP_L4_C_FPAR_8DAY(PkgTheme *pkgTheme_p,char *themeLabel_p);
void l4c_PkgInit_L4_C_MODIS_SetProperties(char *themeLabel_p);


// ----------- l4c_MODIS_logic.c model prototypes -------
int reorg_VIIRS_Qc(int32 nPix,uint8 *tileFparQc_p,uint8 *tileExtraQc_p);

int tile_loop(char *datestr, char *yeardoyString, char *fparTileList[], int ntiles) ;

// <JMG> 2017-02-10T17:07, corrected prototype for ptr <LAJ> 2017-02-01  ADD: 
int32 resample_tile_500m_to_1km(tile_struct *tile_500m, tile_struct *tile_1km) ;
//<LAJ> 2017-02-01  ADD: 
uint8 resample_fpar_window(uint8 *fpar_window4x4, uint8 *qc_window4x4, uint8 *fpar1km, uint8 *qc1km );


int pop_tile_to_1kmease2(tileref_struct *tileref_p, tile_struct *tile_p, ease_struct *ease_p);

int find_corner(tile_struct *tile_p, int *row, int *col);

int get_patch_dims(tile_struct *tile_p, int *istart, int *istop, int *jstart, int *jstop);

// int read_list(char *list, char *fnames[], int *ntiles);

int get_sinusoid_latlon(float64 upleft[], float64 lowright[], float64 lat[], float64 lon[], float64 *dx, float64 *dy);

int parse_filename(char *fname, char *substr, int *x, char str[]);

int get_yeardoy_from_filename(char *fname, char str[]);

void dimswap(uint8 *A);
// <JMG> added 8/19/2014...f
void dimswap_int16(int16 *A) ;
void dimswap_int8(int8 *A);

int  load_tileref(tileref_struct *tileref_p);

void set_land_mask_1km(uint8 mask[], uint8 *fparImg_p, int16 *ndviImg_p, uint8 *fparQcImg_p,int8 *ndviQcImg_p);


// ---------l4c_MODIS_fx.c--------------------------------------
// Utility Functions
int   l4c_strCompare(const void *p1,const void *p2) ;
// string support functions
char *l4c_strGetBaseNamePath(const char *pathStr_P,char *baseName_p,int pathSepCh);
char *l4c_strGetPathPrefix(const char *pathStr_p,char *pathPrefix_p,int pathSepCh);
char *l4c_strExtractExt(char *source,char *dest,const int sep_char);
char *l4c_strExtractRoot(char *source,char *dest,const int sep_char);
char *l4c_to_lower(char *targString);
char *l4c_to_upper(char *targString);

// string table constructor, destructor
char   **l4c_createStringTbl(const int nMembers,const int strWidth);
void     l4c_freeStringTbl(char **strTbl_p,const int nMembers);

char    *l4c_getCtime(char *timBuf,const time_t usersTime);
pid_t    l4c_getPid(pid_t *parents_pid) ;
char    *l4c_GenerateUUID(char *resultUUID_String,const int displayMode);
void    *l4c_rowReversal(float32 *src_p,const int32 nRows,const int32 nCols);
// BIT MANIP routines
uint8        l4c_testByteorder(void);
unsigned int l4c_GetBits(unsigned int xTarget,const unsigned int pStart,const unsigned int nBits);
uint32       l4c_SetBits(int16 startPos, int16 endPos, void *data, size_t dataSize);
uint32       l4c_RetrieveBits(const int16 startPos,const int16 endPos, void *data,size_t dataSize);

// diagnostics
int l4c_calcFrequencies(char *taskLabel_p,uint8 *sourceImg_p,const int32 nElem,FILE *destf);

// ---------l4c_MODIS_io.c--------------------------------------
// I/O for HDF5 and misc
int      l4c_AllocateFparEasegrid(void);

// <JMG> 2021-07-14T17:34, distinguish reads of VIIRs hdfeos5 from hdfeos2 required by MODIS
int      l4c_read_hdfeos2(char *fname_p, tile_struct *tile_p,const int ithTile,const int nTiles);
int      l4c_read_hdfeos5(char *fname_p, tile_struct *tile_p,const int ithTile,const int nTiles);

hid_t    l4c_createHdf5Attribute(const hid_t owner_id,const char *attrLabel,const char *attrValue);

int      l4c_readSmapAncillary(void) ;

uint8    *l4c_NDVI_RescaleConversion(void);

int      l4c_writeSmapOutputs(void);

int      l4c_writeHdf5AttributesToOutput(hid_t file_id);

int      l4c_writeHdf5Dataset(const hid_t out_file_id,char *dsName,const hid_t dsType_id,
                const hid_t dcpl,const hid_t dspace_id,void *dsData_p);

hid_t    l4c_writeNvpAsAttribute(hid_t file_id,const char *keyword,
                const char *attrForceName) ;
                
// <JMG> 2014-08-19, added, cloned from l4c_MET and l4c_DAILY
int l4c_readAndWriteLatLon_1kmCoordVars(void);
                    
// ---------l4c_MODIS_drv.c---------------------------------------------
int l4c_MODIS_Driver(void);

// ---------l4c_MODIS_WriteMetadata.c-----------------------------------
void l4c_Hdf5_AddFileMetadata(void) ;
// adds minimal CF attributes to root level
int l4c_Hdf5_AddFormalMetadata(const hid_t file_id);

// <JMG> 2014-10-14, deprecated l4c_MODIS_metadata.c , used L4CSYS std
// l4c_MODIS_WriteMetadata.c from L4CMDL, L4CMET

// ---------l4c_MODIS_metadata.c----------------------------------------
hid_t l4c_CreateSmapMetadata(hid_t file_id);


// ---------EASEGRID FUNCTION PROTOTYPES ------------------------
// ---------l4c_MODIS_easegrid.c PROTOTYPES ---------------------
int32 l4c_easegrid2_wgs84_fwd(const int32 mapProjectionCode,const float64 lat,const float64 lon,
        float64 *ithRow, float64 *ithColumn);
int32 l4c_easegrid2_wgs84_inv(const int32 mapProjectionCode,float64 ithRow, float64 ithColumn,
        float64 *lat,float64 *lon);


#endif

