SMAP Level 4 Carbon (L4C) Operational Code
==========================================

**Author:** K. Arthur Endsley
**Date:** 2021-07-21

There are four components to the L4C operational code base:

- L4C Framework API: `L4CAPI`
- L4C pre-processor for meteorological driver data: `L4CMET`
- L4C pre-processor for fPAR data: `L4CMOD`
- L4C Model: `L4CMDL`

Each component must be built and installed separately.


Setup and Installation
----------------------

The L4C Framework API, `L4CAPI`, must be built and installed prior to installing any other component. From `L4CSYS/L4CAPI/src`:

```
make
make install
```

However, you should first make sure your build environment is configured correctly (see next section).


Configuring the Build Environment
---------------------------------

First, you must configure the build environment in order to install `L4CSYS` and its dependencies. The following should satisfy dependencies on Ubuntu GNU/Linux 20.04 and later:

```
sudo apt install libuuid1 uuid-dev libhdf4-0 libhdf4-dev libhdf5-dev
```

**NOTE:** If you have installed `libgdal-dev` on your system, it requires `libhdf4alt-dev` instead of `libhdf4-dev`. This causes problems with the existing Makefile, which expects to find `libhdf.so` and `libmfhdf.so`. The solution is to change:

- Change compiler flag `-ldf` to `-ldfalt`
- Change compiler flag `-lmfhdf` to `-lmfhdfalt`

The following environmental variables, with file paths modified according to your system, should be defined in your `.bashrc` file. **Note that it will be necessary to source your `.bashrc` file, i.e., `source ~/.bashrc`, after any changes are made!**

```sh
# Exports for UM L4C Framework API
export HDFBIN=/usr/bin
export HDFLIB=/usr/lib64
export HDFINC=/usr/include
export HDF4INC=$HDFINC
export HDF4LIB=$HDFLIB
export HDF4_INC=$HDF4INC
export HDF4_LIB=$HDF4LIB

export HDF5_ROOT=/usr/local/HDF5_v1.10.6
export HDF5_INC=$HDF5_ROOT/include
export HDF5_LIB=$HDF5_ROOT/lib
export HDF5INC=$HDF5_INC
export HDF5LIB=$HDF5_LIB

export SMAP_ROOT=/anx_lagr4/SMAP/L4C_code/
export L4CSYS_ROOT=$SMAP_ROOT/L4CSYS
export L4CAPI_ROOT=$L4CSYS_ROOT/L4CAPI
export L4C_ROOT=$L4CSYS_ROOT/L4CMDL
export L4C_ANC_ROOT=$L4CSYS_ROOT/ANC
export L4C_MET_ROOT=$L4CSYS_ROOT/L4CMET
export L4C_MODIS_ROOT=$L4CSYS_ROOT/L4CMOD
export FRM_API_ROOT=$L4CAPI_ROOT
export FRM_API_INC=$FRM_API_ROOT/inc
export FRM_API_LIB=$FRM_API_ROOT/lib
```

**To build and install, from the `src` directory:**

```
make
make install
```

**For parallel HDF5 support using MPI, install `libhdf5-mpi-dev` and specifying `CC = mpicc` as the compiler.** However, it seems that the L4C code base is not designed to take advantage of parallel HDF5 support.


Troubleshooting
---------------

**Troubleshooting the build environment.**

If you have trouble locating dependencies on your system (e.g., `/usr/bin/ld: cannot find -ldf`), try using the `ld` utility.

Try calling `ld` in `--verbose` mode, e.g.:

```sh
$ ld -ldf --verbose
```

You'll probably see "attempt to open" and "failed;" the filename it is failing on is the issue, assuming there is no later "succeeded" (it looks in multiple places). Similarly, you can use a scoped `find`, for example:

```sh
find /usr -name hdf5.h
```

**If `frm_api.h` cannot be found.**

```
frm_core.c:20:10: fatal error: frm_api.h: No such file or directory
   20 | #include "frm_api.h"
```

Then check the following environmental variables in your `.bashrc` file: `FRM_API_INC`, `FRM_API_ROOT`, `L4CAPI_ROOT`, `L4CSYS_ROOT`, and `SMAP_ROOT`.

**If `hdf.h` or `hdf5.h` cannot be found.**

```
frm_core.h:166:10: fatal error: hdf.h: No such file or directory
  166 | #include "hdf.h"
```

Check the `HDFINC` or `HDF5INC` environmental variables.
